@@ -327,14 +327,17 @@ sub lookup_enum_documentation($$$$$$$)
   return $docs;
 }
 
-# $strCommentBlock lookup_documentation($strFunctionName, $deprecation_docs, $newin, $objCppfunc)
-# The final objCppfunc parameter is optional.  If passed, it is used to
-# decide if the final C parameter should be omitted if the C++ method
-# has a slot parameter. It is also used for converting C parameter names to
-# C++ parameter names in the documentation, if they differ.
-sub lookup_documentation($$$;$)
+# $strCommentBlock lookup_documentation($strFunctionName, $deprecation_docs,
+#   $newin, $objCppfunc, $errthrow, $voidreturn)
+# The parameters from objCppfunc are optional. If objCppfunc is passed, it is used for
+# - deciding if the final C parameter shall be omitted if the C++ method
+#   has a slot parameter,
+# - converting C parameter names to C++ parameter names in the documentation,
+#   if they differ,
+# - deciding if the @return section shall be omitted.
+sub lookup_documentation($$$;$$$)
 {
-  my ($functionName, $deprecation_docs, $newin, $objCppfunc) = @_;
+  my ($functionName, $deprecation_docs, $newin, $objCppfunc, $errthrow, $voidreturn) = @_;
 
   my $objFunction = $DocsParser::hasharrayFunctions{$functionName};
   if(!$objFunction)
@@ -364,7 +367,11 @@ sub lookup_documentation($$$;$)
   }
 
   my %param_name_mappings = DocsParser::append_parameter_docs($objFunction, \$text, $objCppfunc);
-  DocsParser::append_return_docs($objFunction, \$text);
+  unless ((defined($objCppfunc) && $$objCppfunc{rettype} eq "void") || $voidreturn)
+  {
+    DocsParser::append_return_docs($objFunction, \$text);
+  }
+  DocsParser::add_throws(\$text, $errthrow);
 
   # Convert C parameter names to C++ parameter names where they differ.
   foreach my $key (keys %param_name_mappings)
@@ -674,7 +681,7 @@ sub convert_tags_to_doxygen($)
     # gtk-doc uses @thearg, but doxygen uses @a thearg.
     s" ?\@([a-zA-Z0-9]*(_[a-zA-Z0-9]+)*)_?\b" \@a $1"g;
 
-    # Don't convert Doxygen's $throw, @throws and @param, so these can be used
+    # Don't convert Doxygen's @throw, @throws and @param, so these can be used
     # in the docs_override.xml.
     # Also don't convert @enum and @var which are used for enum documentation.
     s" \@a (throws?|param|enum|var)\b" \@$1"g;
@@ -744,6 +751,24 @@ sub replace_or_add_newin($$)
   }
 }
 
+# void add_throws(\$text, $errthrow)
+# If $errthrow is defined and not empty, and $$text does not contain a @throw,
+# @throws or @exception Doxygen command, add one or more @throws commands.
+sub add_throws($$)
+{
+  my ($text, $errthrow) = @_;
+
+  return if (!defined($errthrow) or $errthrow eq "");
+
+  if (!($$text =~ /[\@\\](throws?|exception)\b/))
+  {
+    # Each comma, not preceded by backslash, creates a new @throws command.
+    $errthrow =~ s/([^\\]),\s*/$1\n\@throws /g;
+    $errthrow =~ s/\\,/,/g; # Delete backslash before comma
+    $$text .= "\n\n\@throws $errthrow";
+  }
+}
+
 # Convert <simplelist> tags to a list of newline-separated elements.
 sub convert_simplelist($)
 {
@@ -781,19 +806,20 @@ sub substitute_identifiers($$)
   {
     # TODO: handle more than one namespace
 
+    # The gtk-doc syntax for links to symbols is described in the gtk-doc manual:
+    # yelp gtk-doc/help/manual/C/index.docbook
+    # then select "Documenting the code" and "Documenting symbols".
+
     # Convert property names to C++.
     # The standard (and correct) gtk-doc way of referring to properties.
     s/(#[A-Z]\w+):([a-z\d-]+)/my $name = "$1::property_$2()"; $name =~ s"-"_"g; "$name";/ge;
-    # This is an incorrect format but widely used so correctly treat as a
-    # property.
+    # This is an incorrect format but widely used so correctly treat as a property.
     s/(\s)::([a-z\d-]+)(\s+property)/my $name = "$1property_$2()$3"; $name =~ s"-"_"g; "$name";/ge;
     # This one catches properties written in the gtk-doc block as for example
-    # '#GtkActivatable::related-action property'.  The correct way to write it
+    # '#GtkActivatable::related-action property'. The correct way to write it
     # would be 'GtkActivatable:related-action' (with a single colon and not
-    # two because the double colons are specifically for signals -- see the
-    # gtk-doc docs:
-    # http://developer.gnome.org/gtk-doc-manual/unstable/documenting_symbols.html.en)
-    # but a few are written with the double colon in the gtk+ docs so this
+    # two because the double colons are specifically for signals)
+    # but a few are written with the double colon in the gtk docs so this
     # protects against those errors.
     s/([A-Z]\w+)::([a-z\d-]+)(\s+property)/my $name = "$1::property_$2()$3"; $name =~ s"-"_"g; "$name";/ge;
 
